"use strict";
/*!
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const container_definitions_1 = require("@fluidframework/container-definitions");
const isomorphic_fetch_1 = __importDefault(require("isomorphic-fetch"));
const utils_1 = require("./utils");
class FluidPackage {
    constructor(codeDetails, packageUrl) {
        this.codeDetails = codeDetails;
        this.packageUrl = packageUrl;
    }
    async resolve() {
        if (this.resolveP === undefined) {
            this.resolveP = this.resolveCore();
        }
        return this.resolveP;
    }
    async resolveCore() {
        let packageJson;
        if (typeof this.codeDetails.package === "string") {
            const response = await isomorphic_fetch_1.default(`${this.packageUrl}/package.json`);
            packageJson = await response.json();
        }
        else {
            packageJson = this.codeDetails.package;
        }
        if (!container_definitions_1.isFluidPackage(packageJson)) {
            throw new Error(`Package ${packageJson.name} not a Fluid module.`);
        }
        const files = packageJson.fluid.browser.umd.files;
        for (let i = 0; i < packageJson.fluid.browser.umd.files.length; i++) {
            if (!files[i].startsWith("http")) {
                files[i] = `${this.packageUrl}/${files[i]}`;
            }
        }
        return {
            config: this.codeDetails.config,
            package: this.codeDetails.package,
            resolvedPackage: packageJson,
            resolvedPackageCacheId: this.packageUrl,
        };
    }
}
/**
 * This code resolver works against cdns that support semantic verioning in the url path of the format:
 * `cdn_base/@package_scope?/package_name@package_version`
 *
 * The `@package_scope?` is optional, and only needed it the package has a scope.
 * The `package_version` can be an npm style semantic version.
 *
 * The `cdn_base` is provided in the config of the Fluid code details, as either a global `config.cdn` property, or
 * a per scope cdn, `config["@package_scope:cdn"]`. A scope specific cdn base will take precedence over
 * the global cdn.
 */
class SemVerCdnCodeResolver {
    constructor() {
        // Cache goes CDN -> package -> entrypoint
        this.fluidPackageCache = new Map();
    }
    async resolveCodeDetails(details) {
        var _a;
        const parsed = utils_1.extractPackageIdentifierDetails(details.package);
        const cdn = (_a = details.config[`@${parsed.scope}:cdn`], (_a !== null && _a !== void 0 ? _a : details.config.cdn));
        const scopePath = parsed.scope !== undefined && parsed.scope.length > 0 ? `@${encodeURI(parsed.scope)}/` : "";
        const packageUrl = parsed.version !== undefined
            ? `${cdn}/${scopePath}${encodeURI(`${parsed.name}@${parsed.version}`)}`
            : `${cdn}/${scopePath}${encodeURI(`${parsed.name}`)}`;
        if (!this.fluidPackageCache.has(packageUrl)) {
            const resolved = new FluidPackage(details, packageUrl);
            this.fluidPackageCache.set(packageUrl, resolved);
        }
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return this.fluidPackageCache.get(packageUrl).resolve();
    }
}
exports.SemVerCdnCodeResolver = SemVerCdnCodeResolver;
//# sourceMappingURL=semVerCdnCodeResolver.js.map